﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class WaveControl : MonoBehaviour
{
    [Header("Ship points")]
    public BoxCollider ShipCenter;
    private List<int> ShipCenterVerts = new List<int>();
    public BoxCollider ShipFront;
    private List<int> ShipFrontVerts = new List<int>();
    public BoxCollider ShipRear;
    private List<int> ShipRearVerts = new List<int>();
    public BoxCollider ShipLeft;
    private List<int> ShipLeftVerts = new List<int>();
    public BoxCollider ShipRight;
    private List<int> ShipRightVerts = new List<int>();

    private Mesh Mesh;
    private Vector3[] Verts;
    private Vector3[] VertNormals;

    [Header("Base Waves")]
    public Texture2D BWTexture;
    public float BWScale = 1;
    private Vector2 BWScrollOffset = new Vector2(0.50f, 0.5f);
    public float BWCurrentHeight = 0.5f;
    public Vector2 BWmapScrollSpeed = new Vector2(0.1f, 0.1f);
    [Header("OverLay Waves")]
    public Texture2D OWTexture;
    public float OWScale = 1;
    private Vector2 OWScrollOffset = new Vector2(0.0f, 0.0f);
    public float OWCurrentHeight = 0.5f;
    public Vector2 OWmapScrollSpeed = new Vector2(0.1f, 0.1f);
    [Space(10)]
    public float CurrentHeight = 0.5f;
    

    /*
    void OnDrawGizmos()
    {
        Gizmos.color = Color.white;
        Mesh = GetComponent<MeshFilter>().mesh;
        Verts = Mesh.vertices;
        for (int i = 0; i < Verts.Length; i++)
            Gizmos.DrawWireSphere(transform.TransformPoint( Verts[i]), 0.01f);
    }
    */


    void Start()
    {
        Mesh = GetComponent<MeshFilter>().sharedMesh;
        Verts = Mesh.vertices;
        VertNormals = Mesh.normals;

        for (int i = 0; i < Verts.Length; i++)
        {
            if (ShipCenter.bounds.Contains(transform.TransformPoint(Verts[i])))
                ShipCenterVerts.Add(i);
            if (ShipFront.bounds.Contains(transform.TransformPoint(Verts[i])))
                ShipFrontVerts.Add(i);
            if (ShipRear.bounds.Contains(transform.TransformPoint(Verts[i])))
                ShipRearVerts.Add(i);
            if (ShipLeft.bounds.Contains(transform.TransformPoint(Verts[i])))
                ShipLeftVerts.Add(i);
            if (ShipRight.bounds.Contains(transform.TransformPoint(Verts[i])))
                ShipRightVerts.Add(i);
        }

        ShipFront.gameObject.SetActive(false);
        ShipRear.gameObject.SetActive(false);
        ShipRight.gameObject.SetActive(false);
        ShipLeft.gameObject.SetActive(false);
    }

    void Update()
    {
        Vector2 lSpeed = BWmapScrollSpeed * Time.deltaTime;
        BWScrollOffset += lSpeed;
        lSpeed = OWmapScrollSpeed * Time.deltaTime;
        OWScrollOffset += lSpeed;

        float _VertHeight = 0;
        for (int i = 0; i < Verts.Length; i++)
        {
            //Verts[i] = new Vector3(Verts[i].x,transform.position.y + (Random.Range(-1,1)),Verts[i].y);
            //Verts[i].y = VertsStart[i].y + Mathf.Cos(Time.time) * 0.01f;
            _VertHeight = 0;
            _VertHeight = BWTexture.GetPixelBilinear((Verts[i].x * BWScale) + BWScrollOffset.x, (Verts[i].z * BWScale) + BWScrollOffset.y).r * BWCurrentHeight;
            _VertHeight += OWTexture.GetPixelBilinear((Verts[i].x * OWScale) + OWScrollOffset.x, (Verts[i].z * OWScale) + OWScrollOffset.y).r * OWCurrentHeight;
            Verts[i].y = _VertHeight;
        }
        Mesh.vertices = Verts;
        
        // height
        float _ShipHeight = 0;
        foreach(int _I in ShipCenterVerts)
            _ShipHeight += transform.TransformPoint(Verts[_I]).y;

        BoatController.Instance.MyHeight = _ShipHeight / ShipCenterVerts.Count;
        /*
        // front
        float _ShipRotPower = 0;
        foreach (int _I in ShipFrontVerts)
            _ShipRotPower += transform.TransformPoint(Verts[_I]).y - ShipFront.transform.position.y;

        _ShipRotPower = _ShipRotPower / ShipFrontVerts.Count;
        Quaternion _Q = Quaternion.Euler(new Vector3(4 * _ShipRotPower, 0, 0) * (Time.deltaTime));
        BoatController.Instance.RotateBoat(_Q);
        
        // rear
        _ShipRotPower = 0;
        foreach (int _I in ShipRearVerts)
            _ShipRotPower += transform.TransformPoint(Verts[_I]).y - ShipRear.transform.position.y;

        _ShipRotPower = _ShipRotPower / ShipRearVerts.Count;
        _Q = Quaternion.Euler(new Vector3(4 * _ShipRotPower, 0, 0) * (Time.deltaTime));
        BoatController.Instance.RotateBoat(_Q);

        // left
        _ShipRotPower = 0;
        foreach (int _I in ShipLeftVerts)
            _ShipRotPower += transform.TransformPoint(Verts[_I]).y - ShipLeft.transform.position.y;

        _ShipRotPower = _ShipRotPower / ShipLeftVerts.Count;
        _Q = Quaternion.Euler(new Vector3(4 * _ShipRotPower, 0, 0) * (Time.deltaTime));
        BoatController.Instance.RotateBoat(_Q);

        // right
        _ShipRotPower = 0;
        foreach (int _I in ShipRightVerts)
            _ShipRotPower += transform.TransformPoint(Verts[_I]).y - ShipRight.transform.position.y;

        _ShipRotPower = _ShipRotPower / ShipRightVerts.Count;
        _Q = Quaternion.Euler(new Vector3(4 * _ShipRotPower, 0, 0) * (Time.deltaTime));
        BoatController.Instance.RotateBoat(_Q);
        */
        //
        Vector3 _Normal01 = CrossDirection(transform.TransformPoint(Verts[ShipFrontVerts[0]]) , transform.TransformPoint(Verts[ShipRightVerts[0]]) , transform.TransformPoint(Verts[ShipRearVerts[0]]) );
        Vector3 _Normal02 = CrossDirection(transform.TransformPoint(Verts[ShipRightVerts[0]]), transform.TransformPoint(Verts[ShipRearVerts[0]]), transform.TransformPoint(Verts[ShipLeftVerts[0]]));
        Vector3 _Normal03 = CrossDirection(transform.TransformPoint(Verts[ShipRearVerts[0]]), transform.TransformPoint(Verts[ShipLeftVerts[0]]), transform.TransformPoint(Verts[ShipFrontVerts[0]]));
        Vector3 _Normal04 = CrossDirection(transform.TransformPoint(Verts[ShipLeftVerts[0]]), transform.TransformPoint(Verts[ShipFrontVerts[0]]), transform.TransformPoint(Verts[ShipRightVerts[0]]));

        Vector3 _AvNormal = ( _Normal01 + _Normal02 + _Normal03 + _Normal04 ) / 4;

        BoatController.Instance.UpVector = _AvNormal;
    }

    Vector3 CrossDirection(Vector3 a, Vector3 b, Vector3 c)
    {
        Vector3 side1 = b - a;
        Vector3 side2 = c - a;
        return Vector3.Cross(side1, side2).normalized;
    }
}	
